import traceback
import re
from pathlib import Path
import time
import subprocess
import logging
import os
from depthai_gate.exceptions import PublicError
from threading import RLock, Thread

class UpdateManager:
    _thread: Thread
    _inprocess: bool
    _progress: float
    _last_message: str
    _mutex: RLock

    def __init__(self):
        self._thread = None
        self._inprocess = False
        self._progress = 0.0
        self._last_message = ''
        self._mutex = RLock()

    @property
    def inprocess(self):
        return self._inprocess
    @property
    def progress(self):
        return self._progress
    @property
    def last_message(self):
        return self._last_message

    def update_thread(self, file_location):
        self._inprocess = True
        self._progress = 0.0
        self._last_message = ''
        try:
            with subprocess.Popen(['mender', '-install', file_location], stdout=subprocess.PIPE) as process:
                while process.poll() is None:
                    line = process.stdout.readline().decode('utf-8')
                    logging.debug(line)
                    # we are looking for a line: x%  ...
                    match = re.search(r'([0-9]+)%', line)
                    if match:
                        self._progress = int(match.group(1)) / 100.0
            os.unlink(file_location)
            self._last_message = 'Done! Rebooting...'
        except Exception as ex:
            traceback.print_exc()
            self._last_message = f'Error updating. Rebooting... Message: {ex}'
            logging.error(self._last_message)
        finally:
            self._inprocess = False
            time.sleep(2) # To give some time for the progress to get through as well
            subprocess.Popen(['/sbin/reboot'])

    def update_thread_mock(self, file_location):
        self._inprocess = True
        for i in range(0,100):
            self._progress = i / 100.0
            time.sleep(0.1)
        self._progress = 1
        self._inprocess = False
        self._last_message = 'Done!'


    def update(self, file_location):
        """Update OS"""

        with self._mutex:
            if self._thread != None:
                if not self._thread.is_alive():
                    self._thread.join()
                else:
                    raise PublicError('Update already in progress')

            # # MOCK function
            # self._thread = Thread(target=self.update_thread_mock, args=(file_location, ))
            self._thread = Thread(target=self.update_thread, args=(file_location, ))
            self._thread.start()

